﻿Option Strict On
Option Explicit On
Imports ESRI.ArcGIS.Geodatabase
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Geometry
Public Class ECMainForm
    Private expDictionary As Dictionary(Of String, String)
    Private descrDictionary As Dictionary(Of String, String)
    Private Sub ECMainForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Try
            Dim CatDictionary As Dictionary(Of String, String) = GetExpressionsCategories()
            TextBox1.Text = String.Empty
            TextBox2.Text = String.Empty
            TextBox3.Text = String.Empty
            opData.Checked = True
            For Each key1 As String In CatDictionary.Keys
                cmbCategory.Items.Add(key1)
            Next
        Catch
            MsgBox(Err.Description)
        End Try
    End Sub
    Private Sub cmbCategory_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbCategory.SelectedIndexChanged
        Try
            cmbExpression.Items.Clear()
            cmbLayer.Items.Clear()
            cmbField.Items.Clear()
            TextBox1.Text = String.Empty
            TextBox2.Text = String.Empty
            TextBox3.Text = String.Empty
            TextBox1.Visible = False
            labPar1.Visible = False
            TextBox2.Visible = False
            labPar2.Visible = False
            TextBox3.Visible = False
            labPar3.Visible = False
            fraSR.Visible = False
            Select Case cmbCategory.Text
                Case "Geometry"
                    labWarn.Visible = False
                    expDictionary = GetGeometryExpressions()
                    descrDictionary = GetGeometryExpressionsDescription()
                Case "Draw"
                    labWarn.Visible = False
                    expDictionary = GetDrawExpressions()
                    descrDictionary = GetDrawExpressionsDescription()
                Case "Edit"
                    labWarn.Visible = True
                    expDictionary = GetEditExpressions()
                    descrDictionary = GetEditExpressionsDescription()
            End Select
            If Not expDictionary Is Nothing Then
                For Each key1 As String In expDictionary.Keys
                    cmbExpression.Items.Add(key1)
                Next
            End If

        Catch
            MsgBox(Err.Description)
        End Try
    End Sub
    Private Sub cmbExpression_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbExpression.SelectedIndexChanged

        Try
            cmbLayer.Items.Clear()
            cmbField.Items.Clear()
            opData.Checked = True
            TextBox1.Text = String.Empty
            TextBox2.Text = String.Empty
            TextBox3.Text = String.Empty
            Dim sExp As String = cmbExpression.Text
            If expDictionary.ContainsKey(sExp) Then
                If descrDictionary.ContainsKey(sExp) Then
                    labDescription.Text = descrDictionary.Item(sExp)
                End If
                Dim sData As String = expDictionary.Item(sExp)
                Dim sDataDetail() As String = Split(sData, "|")
                If UBound(sDataDetail) < 3 Then
                    MsgBox("Incorrect expression data for " & sExp)
                    Exit Sub
                End If
                Dim sLayerType = sDataDetail(0)
                Dim pColl As Collection = EC_GetFeatureLayers(sLayerType)
                Dim sLN As String
                Dim i As Integer
                If Not pColl Is Nothing Then
                    For i = 1 To pColl.Count
                        sLN = pColl.Item(i).ToString
                        cmbLayer.Items.Add(sLN)
                    Next
                End If
                Dim sSR As String = sDataDetail(2)
                If sSR = "Y" Then
                    fraSR.Visible = True
                Else
                    fraSR.Visible = False
                End If
                If UBound(sDataDetail) >= 3 Then
                    Dim iNumPars As Short = CShort(sDataDetail(3))
                    If iNumPars > 0 Then
                        Select Case iNumPars

                            Case 1
                                labPar1.Text = sDataDetail(4)
                                TextBox1.Visible = True
                                labPar1.Visible = True
                                TextBox2.Visible = False
                                labPar2.Visible = False
                                TextBox3.Visible = False
                                labPar3.Visible = False
                            Case 2
                                labPar1.Text = sDataDetail(4)
                                labPar2.Text = sDataDetail(5)
                                TextBox1.Visible = True
                                labPar1.Visible = True
                                TextBox2.Visible = True
                                labPar2.Visible = True
                                TextBox3.Visible = False
                                labPar3.Visible = False
                            Case 3
                                labPar1.Text = sDataDetail(4)
                                labPar2.Text = sDataDetail(5)
                                labPar2.Text = sDataDetail(6)
                                TextBox1.Visible = True
                                labPar1.Visible = True
                                TextBox2.Visible = True
                                labPar2.Visible = True
                                TextBox3.Visible = True
                                labPar3.Visible = True
                            Case Else
                                MsgBox("Incorrect number parameters " & sExp)
                                Exit Sub
                        End Select
                    Else
                        TextBox1.Visible = False
                        labPar1.Visible = False
                        TextBox2.Visible = False
                        labPar2.Visible = False
                        TextBox3.Visible = False
                        labPar3.Visible = False
                    End If
                End If
            End If
        Catch
            MsgBox(Err.Description)
        End Try
    End Sub



    Private Sub cmbLayer_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbLayer.SelectedIndexChanged
        Try
            cmbField.Items.Clear()
            Dim sLN As String = cmbLayer.Text
            Dim pFL As IFeatureLayer = EC_GetLayer(sLN)
            If pFL Is Nothing Then
                MsgBox("Could not find feature layer " & sLN)
                Exit Sub
            End If
            Dim sExp As String = cmbExpression.Text
            If expDictionary.ContainsKey(sExp) Then
                Dim sData As String = expDictionary.Item(sExp)
                Dim sDataDetail() As String = Split(sData, "|")
                If UBound(sDataDetail) < 3 Then
                    MsgBox("Incorrect expression data for " & sExp)
                    Exit Sub
                End If
                Dim sFieldType = sDataDetail(1)
                Dim pFC As IFeatureClass = pFL.FeatureClass
                Dim pFields As IFields = pFC.Fields
                Dim pField As IField = Nothing
                Dim i As Integer
                cmbField.Items.Clear()
                For i = 0 To pFields.FieldCount - 1
                    pField = pFields.Field(i)
                    Select Case sFieldType
                        Case "Number"
                            If pField.Type < 4 Then
                                cmbField.Items.Add(pField.Name)
                            End If
                        Case "String"
                            If pField.Type = 4 Then
                                cmbField.Items.Add(pField.Name)
                            End If
                        Case "Date"
                            If pField.Type = esriFieldType.esriFieldTypeDate Then
                                cmbField.Items.Add(pField.Name)
                            End If
                        Case Else
                    End Select

                Next i
            End If
        Catch
            MsgBox(Err.Description)
        End Try
    End Sub

    Private Sub cmdCalculate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCalculate.Click
        Dim bEditing As Boolean = False
        Dim sExpression As String = cmbExpression.Text
        Dim pEditor As ESRI.ArcGIS.Editor.IEditor = EC_GetEditor()
        Try
            If expDictionary Is Nothing Then
                MsgBox("Select an expression")
                Exit Sub
            End If
            If Not expDictionary.ContainsKey(sExpression) Then
                MsgBox("Select an expression")
                Exit Sub
            End If
            Dim pFL As IFeatureLayer = EC_GetLayer(cmbLayer.Text)
            If pFL Is Nothing Then
                MsgBox("Select a feature layer")
                Exit Sub
            End If
            Dim sField As String = cmbField.Text
            
            If pFL.FeatureClass.FindField(sField) = -1 Then
                Select Case cmbCategory.Text
                    Case "Geometry"
                        MsgBox("Select a field")
                        Exit Sub
                    Case "Draw"
                        MsgBox("The status will not be recorded in the attribute table")
                    Case "Edit"
                        MsgBox("The status will not be recorded in the attribute table")
                End Select
            End If
            Dim pEditLayers As ESRI.ArcGIS.Editor.IEditLayers = CType(pEditor, ESRI.ArcGIS.Editor.IEditLayers)
            Dim sData As String = expDictionary.Item(sExpression)
            Dim sDataDetail() As String = Split(sData, "|")
            If UBound(sDataDetail) < 3 Then
                MsgBox("Incorrect expression data for " & sExpression)
                Exit Sub
            End If
            Dim pGDS As IGeoDataset
            pGDS = CType(pFL, IGeoDataset)
            Dim pDataSref As ISpatialReference = pGDS.SpatialReference
            Dim pMapSref As ISpatialReference = My.ArcMap.Document.FocusMap.SpatialReference
            params.DataSref = pDataSref
            params.MapSref = pMapSref
            params.SrefFromData = opData.Checked
            params.Par1 = TextBox1.Text
            params.Par2 = TextBox2.Text
            params.Par3 = TextBox2.Text
            If pEditor.EditState = ESRI.ArcGIS.Editor.esriEditState.esriStateEditing Then
                If pEditLayers.IsEditable(pFL) Then
                    pEditor.StartOperation()
                    bEditing = True
                End If
            End If
            Dim bDone As Boolean
            Select Case cmbCategory.Text
                Case "Geometry"
                    bDone = EC_CalculateG(pFL, sField, sExpression)
                Case "Draw"
                    bDone = EC_CalculateD(pFL, sField, sExpression)
                Case "Edit"
                    bDone = EC_CalculateE(pFL, sField, sExpression)
            End Select


            If bDone Then
                Dim pTW As ITableWindow = New TableWindow
                Dim pTWE As ITableWindow = pTW.FindViaFeatureLayer(pFL, False)
                If Not pTWE Is Nothing Then
                    pTWE.Refresh()
                End If
            End If
        Catch ex As Exception
            MsgBox("Error :" & ex.Message)
        Finally
            If bEditing Then
                pEditor.StopOperation(sExpression)
            End If
        End Try
    End Sub

    
End Class